from collections import defaultdict

class StyleVariable:
    def __init__(self):
        self.value = None

    def __call__(self, value):
        self.value = value
        return self

    def __str__(self):
        return str(self.value)


class StyleVariableStorage:
    def __getattr__(self, item):
        if item in self.__dict__:
            return self.__dict__[item]
        else:
            self.__dict__[item] = StyleVariable()
            return self.__dict__[item]

    def __setattr__(self, key, value):
        if key in self.__dict__:
            self.__dict__[key](value)
        else:
            self.__dict__[key] = StyleVariable()
            self.__dict__[key](value)

class joinable_dict(dict):
    """ A slightly changed dict that allows extending values contained using += """
    def __iadd__(self, other):
        self = joinable_dict(**self, **other)
        return self


class Style(defaultdict):
    def __init__(self):
        super().__init__(lambda: joinable_dict())
        self.variable = StyleVariableStorage()


    def __str__(self):
        out = ''
        tab = '  '

        def build_rules(selector, attributes):
            out = selector + ' {\n' + tab + f';\n{tab}'.join(f'{k.replace("_", "-")}: {v}' for k, v in attributes.items() if type(v) is not dict) + ';\n}\n\n'
            out += '\n'.join(build_rules(selector + ' ' + k, v) for k, v in attributes.items() if type(v) is dict)
            return out

        for selector, attributes in self.items():
            out += build_rules(selector, attributes)
        return out

    def list_variables(self):
        return [x for x in dir(self.variable) if not x.startswith('_')]

    def __setitem__(self, key, value):
        """ This method overload allows for += assignment to simpply add values to the contained dict"""
        super().__setitem__(key, joinable_dict(value))

# There is an issue with asignment not resetting. using = multiple times just appends


import re
import ast
from setuptools import setup


_version_re = re.compile(r'__version__\s+=\s+(.*)')


with open('pyno/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))


setup(
    name='pyno',
    author='Jackie Vincent Larsen',
    author_email='jackie.v.larsen@gmail.com',
    version=version,
    #url='https://github.com/jVinc/pyno',
    description='A framework for building html, svg and other structured text in Python',
    keywords=['web-development', 'web'],
    packages=['pyno'],
    install_requires=[],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3'
    ],
)


import tempfile
import webbrowser
import os
import __main__
from bs4 import BeautifulSoup

def html_prettify(html):
    return (BeautifulSoup(str(html), 'html.parser').prettify())


def browser_preview(string_content, dir=None, prettify=True):  # pragma: no cover
    """ Creates a temporary html file with the string_content, and opens it in a browser
    The script then waits for user input and when entered cleans up the temporary file

    set dir=False to use system default location for temporary file.
        dir=None puts file in current directory, to allow relative links to work in testing

    prettify sets if the output should be pretty parsed html or just straight output. Defaults to True, which uses
    BeautifulSoup.prettify to structure html in a nice human readable format
    """

    if prettify:
        convert = html_prettify
    else:
        convert = str
    with tempfile.NamedTemporaryFile(suffix='.html',
                                     delete=True,
                                     dir=os.path.dirname(__main__.__file__) if dir is None
                                     else (None if dir is False else dir)) as file:
        file.write(convert(string_content).encode('utf-8'))
        file.flush()
        print(r'file:\\' + file.name)
        webbrowser.open(r'file:///' + file.name.replace('\\', '/'))
        input("Press Enter to end script and clean up temporary file...")
